@@ -27,6 +27,7 @@
 #include <X11/Xatom.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <libwnck/screen.h>
 
 G_BEGIN_DECLS
 
@@ -98,21 +99,24 @@ void _wnck_iconify   (Screen *screen,
 void _wnck_deiconify (Screen *screen,
                       Window  xwindow);
 
-void _wnck_close     (Screen *screen,
-		      Window  xwindow,
-		      Time    timestamp);
+void _wnck_close (WnckScreen *screen,
+                  Window      xwindow,
+                  Time        timestamp);
+
+void _wnck_change_state (WnckScreen *screen,
+                         Window      xwindow,
+                         gboolean    add,
+                         Atom        state1,
+                         Atom        state2);
+
+void _wnck_change_workspace (WnckScreen *screen,
+                             Window      xwindow,
+                             int         new_space);
+
+void _wnck_activate (WnckScreen *screen,
+                     Window      xwindow,
+                     Time        timestamp);
 
-void _wnck_change_state     (Screen  *screen,
-			     Window   xwindow,
-                             gboolean add,
-                             Atom     state1,
-                             Atom     state2);
-void _wnck_change_workspace (Screen  *screen,
-			     Window   xwindow,
-                             int      new_space);
-void _wnck_activate         (Screen  *screen,
-                             Window   xwindow,
-                             Time     timestamp);
 void _wnck_activate_workspace (Screen *screen,
                                int     new_active_space,
                                Time    timestamp);
@@ -146,11 +150,11 @@ int    _wnck_select_input     (Screen  *screen,
                                int      mask,
                                gboolean update);
 
-void   _wnck_keyboard_move    (Screen *screen,
-                               Window  xwindow);
+void _wnck_keyboard_move (WnckScreen *screen,
+                          Window      xwindow);
 
-void   _wnck_keyboard_size    (Screen *screen,
-                               Window  xwindow);
+void _wnck_keyboard_size (WnckScreen *screen,
+                          Window      xwindow);
 
 void _wnck_toggle_showing_desktop (Screen  *screen,
                                    gboolean show);
@@ -166,15 +170,16 @@ void           _wnck_icon_cache_set_want_fallback    (WnckIconCache *icon_cache,
                                                       gboolean       setting);
 gboolean       _wnck_icon_cache_get_is_fallback      (WnckIconCache *icon_cache);
 
-gboolean _wnck_read_icons         (Screen         *screen,
-                                   Window          xwindow,
-                                   WnckIconCache  *icon_cache,
-                                   GdkPixbuf     **iconp,
-                                   int             ideal_width,
-                                   int             ideal_height,
-                                   GdkPixbuf     **mini_iconp,
-                                   int             ideal_mini_width,
-                                   int             ideal_mini_height);
+gboolean _wnck_read_icons (WnckScreen     *screen,
+                           Window          xwindow,
+                           WnckIconCache  *icon_cache,
+                           GdkPixbuf     **iconp,
+                           int             ideal_width,
+                           int             ideal_height,
+                           GdkPixbuf     **mini_iconp,
+                           int             ideal_mini_width,
+                           int             ideal_mini_height);
+
 void _wnck_get_fallback_icons (GdkPixbuf     **iconp,
                                int             ideal_width,
                                int             ideal_height,
